@@ -22,11 +22,13 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
+import apt
 import apt_pkg
 from hashlib import md5
 import re
 import os
 import glob
+import inspect
 import sys
 import shutil
 import subprocess
@@ -42,14 +44,14 @@ from ConfigParser import ConfigParser
 from gettext import gettext as _
 from urlparse import urlparse
 
-from AptAuth import AptAuth
-import softwareproperties
 import aptsources
 import aptsources.distro
+import softwareproperties
+
+from AptAuth import AptAuth
 from aptsources.sourceslist import SourcesList, SourceEntry
 from ppa import AddPPASigningKeyThread, expand_ppa_line
 
-
 class SoftwareProperties(object):
 
   # known (whitelisted) channels
@@ -59,21 +61,25 @@ class SoftwareProperties(object):
   RELEASE_UPGRADES_CONF = "/etc/update-manager/release-upgrades"
   #RELEASE_UPGRADES_CONF = "/tmp/release-upgrades"
   (
-    RELEASE_UPGRADES_NEVER,
     RELEASE_UPGRADES_NORMAL,
-    RELEASE_UPGRADES_LTS
+    RELEASE_UPGRADES_LTS,
+    RELEASE_UPGRADES_NEVER
   ) = range(3)
   release_upgrades_policy_map = {
-    RELEASE_UPGRADES_NEVER  : 'never',
     RELEASE_UPGRADES_NORMAL : 'normal',
     RELEASE_UPGRADES_LTS    : 'lts',
+    RELEASE_UPGRADES_NEVER  : 'never',
   }
   
-  def __init__(self, datadir=None, options=None):
+  def __init__(self, datadir=None, options=None, rootdir="/"):
     """ Provides the core functionality to configure the used software 
         repositories, the corresponding authentication keys and 
         update automation """
-    self.popconfile = "/etc/popularity-contest.conf"
+    self.popconfile = rootdir+"/etc/popularity-contest.conf"
+
+    self.rootdir = rootdir
+    if rootdir != "/":
+      apt_pkg.config.set("Dir", rootdir)
 
     # FIXME: some saner way is needed here
     if datadir == None:
@@ -97,7 +103,7 @@ class SoftwareProperties(object):
     self.custom_mirrors= []
 
     # apt-key stuff
-    self.apt_key = AptAuth()
+    self.apt_key = AptAuth(rootdir=rootdir)
 
     atexit.register(self.wait_for_threads)
 
@@ -178,9 +184,9 @@ class SoftwareProperties(object):
   def get_release_upgrades_policy(self):
     """
     return the release upgrade policy:
-     RELASE_UPGRADE_NEVER
-     RELASE_UPGRADE_NORMAL
-     RELASE_UPGRADE_LTS
+     RELEASE_UPGRADES_NORMAL,
+     RELEASE_UPGRADES_LTS,
+     RELEASE_UPGRADES_NEVER
     """
     # default (if no option is set) is NORMAL
     if not os.path.exists(self.RELEASE_UPGRADES_CONF):
@@ -197,9 +203,9 @@ class SoftwareProperties(object):
   def set_release_upgrades_policy(self, i):
     """
     set the release upgrade policy:
-     RELASE_UPGRADE_NEVER
-     RELASE_UPGRADE_NORMAL
-     RELASE_UPGRADE_LTS
+     RELEASE_UPGRADES_NORMAL,
+     RELEASE_UPGRADES_LTS,
+     RELEASE_UPGRADES_NEVER
      """
     # we are note using ConfigParser.write() as it removes comments
     if not os.path.exists(self.RELEASE_UPGRADES_CONF):
@@ -319,6 +325,13 @@ class SoftwareProperties(object):
         print source.str()
     print "END SOURCES.LIST\n"
 
+  def change_main_download_server(self, server):
+    """ change the main download server """
+    self.distro.default_server = server
+    res = self.distro.change_server(server)
+    self.set_modified_sourceslist()
+    return res
+
   def enable_component(self, comp):
     """Enable a component of the distro"""
     self.distro.enable_component(comp) 
@@ -329,8 +342,16 @@ class SoftwareProperties(object):
     self.distro.disable_component(comp) 
     self.set_modified_sourceslist()
 
+  def _find_template_from_string(self, name):
+    for template in self.distro.source_template.children:
+      if template.name == name:
+        return template
+
   def disable_child_source(self, template):
     """Enable a child repo of the distribution main repository"""
+    if isinstance(template, str):
+      template = self._find_template_from_string(template)
+
     for source in self.distro.child_sources:
         if source.template == template:
             self.sourceslist.remove(source)
@@ -341,6 +362,9 @@ class SoftwareProperties(object):
 
   def enable_child_source(self, template):
     """Enable a child repo of the distribution main repository"""
+    if isinstance(template, str):
+      template = self._find_template_from_string(template)
+
     # Use the currently selected mirror only if the child source
     # did not override the server
     if template.base_uri == None:
@@ -395,12 +419,30 @@ class SoftwareProperties(object):
         source_bkp = SourceEntry(line=source.line,file=source.file)
         self.sourceslist_backup.append(source_bkp)
   
+  def _find_source_from_string(self, line):
+    for source in self.sourceslist.list:
+      if str(source) == line:
+        return source
+    return None
+
   def toggle_source_use(self, source):
     """Enable or disable the selected channel"""
     #FIXME cdroms need to disable the comps in the childs and sources
+    if isinstance(source, str):
+      source = self._find_source_from_string(source)
     source.disabled = not source.disabled
     self.set_modified_sourceslist()
 
+  def replace_source_entry(self, old_entry, new_entry):
+    # find and replace, then write
+    for (index, entry) in enumerate(self.sourceslist.list):
+      if str(entry) == old_entry:
+        file = self.sourceslist.list[index].file
+        self.sourceslist.list[index] = SourceEntry(new_entry, file)
+        self.set_modified_sourceslist()
+        return True
+    return False
+
   def revert(self):
     """Revert all settings to the state when software-properties 
        was launched"""
@@ -540,9 +582,9 @@ class SoftwareProperties(object):
   def write_config(self):
     """Write the current apt configuration to file"""
     # update the adept file as well if it is there
-    conffiles = ["/etc/apt/apt.conf.d/10periodic",
-                 "/etc/apt/apt.conf.d/20auto-upgrades",
-                 "/etc/apt/apt.conf.d/15adept-periodic-update"]
+    conffiles = [self.rootdir+"/etc/apt/apt.conf.d/10periodic",
+                 self.rootdir+"/etc/apt/apt.conf.d/20auto-upgrades",
+                 self.rootdir+"/etc/apt/apt.conf.d/15adept-periodic-update"]
 
     # check (beforehand) if one exists, if not create one
     for f in conffiles:
@@ -616,7 +658,7 @@ class SoftwareProperties(object):
     # FIXME: abstract this all alway into the ppa.py file
     parsed_uri = urlparse(SourceEntry(srcline).uri)
     if parsed_uri.netloc == 'ppa.launchpad.net':
-      worker = AddPPASigningKeyThread(parsed_uri.path)
+      worker = AddPPASigningKeyThread(parsed_uri.path, self.options and self.options.keyserver)
       worker.start()
 
   def update_interface(self):
@@ -659,6 +701,9 @@ class SoftwareProperties(object):
 
   def remove_source(self, source):
     """Remove the given source"""
+    # first find the source object if we got a string
+    if isinstance(source, str):
+      source = self._find_source_from_string(source)
     # if its a sources.list.d file and it contains only a single line
     # (the line that we just remove), then all references to that
     # file are gone and the file is not saved. we work around that
@@ -675,7 +720,9 @@ class SoftwareProperties(object):
     if not os.path.exists(path):
         return False
     try:
-        return self.apt_key.add(path)
+        res = self.apt_key.add(path)
+        self.KeysModified()
+        return res
     except:
         return False
 
@@ -686,14 +733,23 @@ class SoftwareProperties(object):
     tmp.flush()
     return self.add_key(tmp.name)
 
-  def remove_key(self, fingerprint):
+  def remove_key(self, keyid):
     """Remove a gnupg key from the list of trusted software vendors"""
     try:
-        self.apt_key.rm(fingerprint)
+        self.apt_key.rm(keyid)
+        self.KeysModified()
         return True
     except:
         return False
 
+  def update_keys(self):
+    """ Run apt-key update """
+    try:
+      self.apt_key.update()
+      self.KeysModified()
+      return True
+    except:
+      return False
 
 
 if __name__ == "__main__":
