@@ -21,7 +21,10 @@
 #  USA
 
 # Level of the update automation
-(UPDATE_MANUAL, UPDATE_NOTIFY, UPDATE_DOWNLOAD, UPDATE_INST_SEC) = range(4)
+(UPDATE_MANUAL, 
+ UPDATE_NOTIFY, 
+ UPDATE_DOWNLOAD, 
+ UPDATE_INST_SEC) = range(4)
 
 # Provide more readable synonyms for apt configuration options
 CONF_MAP = {
