@@ -149,6 +149,10 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def init_release_upgrades(self):
     " setup the widgets that allow configuring the release upgrades "
     i = self.get_release_upgrades_policy()
+
+    # FIXME: hack for LP: #944876
+    i = (i + 1) % 3
+
     self.userinterface.combobox_release_upgrades.setCurrentIndex(i)
     kapp.connect(self.userinterface.combobox_release_upgrades, SIGNAL("currentIndexChanged(int)"), self.on_combobox_release_upgrades_changed)
 
@@ -395,6 +399,10 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def on_combobox_release_upgrades_changed(self, combobox):
     """ set the release upgrades policy """
     i = self.userinterface.combobox_release_upgrades.currentIndex()
+
+    # FIXME: hack for LP: #944876
+    i = (i - 1) % 3
+
     self.set_release_upgrades_policy(i)
 
   def on_combobox_server_changed(self, combobox):
@@ -445,14 +453,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     """
     tickBox = kapp.sender()
     name = str(tickBox.objectName())
-    for template in self.distro.source_template.children:
-        if name == template.name:
-            aptTemplate = template
 
     if tickBox.checkState() == Qt.Checked:
-        self.enable_child_source(aptTemplate)
+        self.enable_child_source(name)
     elif tickBox.checkState() == Qt.Unchecked:
-        self.disable_child_source(aptTemplate)
+        self.disable_child_source(name)
     # no way to set back to mixed state so turn off tristate
     tickBox.setTristate(False)
 
